@@ -95,9 +95,6 @@
 G_DEFINE_QUARK (autoar-extractor, autoar_extractor)
 
 #define BUFFER_SIZE (64 * 1024)
-#define NOT_AN_ARCHIVE_ERRNO 2013
-#define EMPTY_ARCHIVE_ERRNO 2014
-#define INCORRECT_PASSPHRASE_ERRNO 2015
 
 typedef struct _GFileAndInfo GFileAndInfo;
 
@@ -992,11 +989,13 @@ autoar_extractor_do_write_entry (AutoarExtractor      *self,
   {
     GFile *parent;
     parent = g_file_get_parent (dest);
-    if (parent && !g_file_query_exists (parent, self->cancellable))
-      g_file_make_directory_with_parents (parent,
-                                          self->cancellable,
-                                          NULL);
-    g_object_unref (parent);
+    if (parent) {
+      if (!g_file_query_exists (parent, self->cancellable))
+        g_file_make_directory_with_parents (parent,
+                                            self->cancellable,
+                                            NULL);
+      g_object_unref (parent);
+    }
   }
 
   info = g_file_info_new ();
@@ -1172,12 +1171,9 @@ autoar_extractor_do_write_entry (AutoarExtractor      *self,
               autoar_extractor_signal_progress (self);
             }
 
-            if (r == ARCHIVE_FAILED) {
+            if (r != ARCHIVE_EOF) {
               if (self->error == NULL) {
-                self->error = g_error_new (AUTOAR_EXTRACTOR_ERROR,
-                                           INCORRECT_PASSPHRASE_ERRNO,
-                                           "%s",
-                                           archive_error_string (a));
+                self->error = autoar_common_g_error_new_a (a, NULL);
               }
               g_output_stream_close (ostream, self->cancellable, NULL);
               g_object_unref (ostream);
@@ -1222,8 +1218,7 @@ autoar_extractor_do_write_entry (AutoarExtractor      *self,
         g_array_append_val (self->extracted_dir_list, fileandinfo);
 
         /* Unset folder permissions for now to be sure it is writable. */
-        g_file_info_set_attribute (info, G_FILE_ATTRIBUTE_UNIX_MODE,
-                                   G_FILE_ATTRIBUTE_TYPE_INVALID, NULL);
+        g_file_info_remove_attribute (info, G_FILE_ATTRIBUTE_UNIX_MODE);
       }
       break;
     case AE_IFLNK:
@@ -1658,18 +1653,16 @@ autoar_extractor_step_scan_toplevel (AutoarExtractor *self)
     r = libarchive_create_read_object (TRUE, self, &a);
     if (r != ARCHIVE_OK) {
       if (self->error == NULL)
-        self->error = autoar_common_g_error_new_a (a, self->source_basename);
+        self->error = autoar_common_g_error_new_a (a, NULL);
       return;
     } else if (archive_filter_count (a) <= 1){
       /* If we only use raw format and filter count is one, libarchive will
        * not do anything except for just copying the source file. We do not
        * want this thing to happen because it does unnecesssary copying. */
       if (self->error == NULL)
-        self->error = g_error_new (AUTOAR_EXTRACTOR_ERROR,
-                                        NOT_AN_ARCHIVE_ERRNO,
-                                        "\'%s\': %s",
-                                        self->source_basename,
-                                        "not an archive");
+        self->error = g_error_new_literal (AUTOAR_EXTRACTOR_ERROR,
+                                           AUTOAR_NOT_AN_ARCHIVE_ERRNO,
+                                           "not an archive");
       return;
     }
     self->use_raw_format = TRUE;
@@ -1688,11 +1681,24 @@ autoar_extractor_step_scan_toplevel (AutoarExtractor *self)
       return;
     }
 
-    if (archive_entry_is_encrypted (entry)) {
+    /* The password is requested only for the ZIP format to avoid showing
+     * password prompt for 7ZIP/RAR, where archive_entry_is_encrypted resp.
+     * archive_entry_is_metadata_encrypted returns TRUE, but followup
+     * archive_read_data_block resp. archive_read_next_header call leads to
+     * an error. See https://github.com/libarchive/libarchive/issues/1662.
+     */
+    if (archive_entry_is_encrypted (entry) &&
+        archive_format (a) == ARCHIVE_FORMAT_ZIP) {
       autoar_extractor_request_passphrase (self);
       if (g_cancellable_is_cancelled (self->cancellable)) {
         archive_read_free (a);
         return;
+      } else if (self->passphrase == NULL) {
+        self->error = g_error_new_literal (AUTOAR_EXTRACTOR_ERROR,
+                                           AUTOAR_PASSPHRASE_REQUIRED_ERRNO,
+                                           "A passphrase is required");
+        archive_read_free (a);
+        return;
       }
     }
 
@@ -1725,22 +1731,19 @@ autoar_extractor_step_scan_toplevel (AutoarExtractor *self)
     archive_read_data_skip (a);
   }
 
-  if (self->files_list == NULL) {
+  if (r != ARCHIVE_EOF) {
     if (self->error == NULL) {
-      self->error = g_error_new (AUTOAR_EXTRACTOR_ERROR,
-                                      EMPTY_ARCHIVE_ERRNO,
-                                      "\'%s\': %s",
-                                      self->source_basename,
-                                      "empty archive");
+      self->error = autoar_common_g_error_new_a (a, NULL);
     }
     archive_read_free (a);
     return;
   }
 
-  if (r != ARCHIVE_EOF) {
+  if (self->files_list == NULL) {
     if (self->error == NULL) {
-      self->error =
-        autoar_common_g_error_new_a (a, self->source_basename);
+      self->error = g_error_new_literal (AUTOAR_EXTRACTOR_ERROR,
+                                         AUTOAR_EMPTY_ARCHIVE_ERRNO,
+                                         "empty archive");
     }
     archive_read_free (a);
     return;
@@ -1877,8 +1880,7 @@ autoar_extractor_step_extract (AutoarExtractor *self) {
   r = libarchive_create_read_object (self->use_raw_format, self, &a);
   if (r != ARCHIVE_OK) {
     if (self->error == NULL) {
-      self->error =
-        autoar_common_g_error_new_a (a, self->source_basename);
+      self->error = autoar_common_g_error_new_a (a, NULL);
     }
     archive_read_free (a);
     return;
@@ -1999,8 +2001,7 @@ autoar_extractor_step_extract (AutoarExtractor *self) {
 
   if (r != ARCHIVE_EOF) {
     if (self->error == NULL) {
-      self->error =
-        autoar_common_g_error_new_a (a, self->source_basename);
+      self->error = autoar_common_g_error_new_a (a, NULL);
     }
     archive_read_free (a);
     return;
@@ -2130,6 +2131,25 @@ autoar_extractor_start_async_thread (GTask        *task,
   g_object_unref (task);
 }
 
+/**
+ * autoar_extractor_set_passphrase:
+ * @self: an #AutoarExtractor object
+ * @passphrase: a string with the passphrase to use
+ *
+ * Sets a passphrase to use if the file to decompress is passphrase-protected.
+ **/
+void
+autoar_extractor_set_passphrase (AutoarExtractor *self,
+                                 const gchar     *passphrase)
+{
+  g_return_if_fail (AUTOAR_IS_EXTRACTOR (self));
+  g_return_if_fail (passphrase != NULL);
+
+  self->passphrase_requested = TRUE;
+
+  g_free (self->passphrase);
+  self->passphrase = g_strdup (passphrase);
+}
 
 /**
  * autoar_extractor_start_async:
